#ifndef ENGINE_H
#define ENGINE_H

/**
 * Engine.h: a simple engine.
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include <boost/scoped_ptr.hpp>
#include <string>

namespace blurt
{
	class Raytracer;
	class Scene;
};

namespace platform { class Screen; };


/**
 * A simple Engine.
 */
class Engine
{
	public:
		~Engine();

		void run(int argc, char** argv);
		static Engine& get();
		const blurt::Scene& getScene() const {return *scene;}
		blurt::Scene& getScene() {return *scene;}
		void saveScene(const std::string& path) const;

	private:
		Engine();
		bool setup(int argc, char** argv);
		void tearDown();
		void update();

		// returns false if the cmd-line contained invalid arguments
		bool parseCommandLine(int argc, char** argv);

		boost::scoped_ptr<blurt::Raytracer> raytracer;
		boost::scoped_ptr<blurt::Scene> scene;
		boost::scoped_ptr<platform::Screen> screen;
		bool isRunning;
		int supersampling;
		int screenWidth;
		int screenHeight;
		std::string inputfile;
		bool makeScreenshot;
		bool showFps;
		int recursionDepth;
		bool renderOnlyOneFrame;
		bool doRender;
		std::string videofile;
};

#endif
