#include "AxisAlignedBoundingBox.h"
#include "Ray.h"
#include "defaultsettings.h"

using namespace blurt;

blurt::AxisAlignedBoundingBox::AxisAlignedBoundingBox
                  (const Vector3D& p1, const Vector3D& p2)
	: pos1(p1),
	  pos2(p2)
{
}

blurt::AxisAlignedBoundingBox::~AxisAlignedBoundingBox()
{
}

bool blurt::AxisAlignedBoundingBox::contains(const Vector3D& point) const
{
	return (point[0] > pos1[0]) && (point[0] < pos2[0])
	&& (point[1] > pos1[1]) && (point[1] < pos2[1])
	&& (point[2] > pos1[2]) && (point[2] < pos2[2]);
}


bool blurt::AxisAlignedBoundingBox::intersectsRay(const Ray& ray,
                                                       float* distance) const
{
	float dist[6];
	Vector3D ip[6];
	const Vector3D& d = ray.getDirection();
	const Vector3D& o = ray.getOrigin();
	bool retval = false;
	for ( int i = 0; i < 6; ++i)
		dist[i] = -1;

	if (d[0])
	{
		float rc = 1.0f / d[0];
		dist[0] = (pos1[0] - o[0]) * rc;
		dist[3] = (pos2[0] - o[0]) * rc;
	}
	if (d[1])
	{
		float rc = 1.0f / d[1];
		dist[1] = (pos1[1] - o[1]) * rc;
		dist[4] = (pos2[1] - o[1]) * rc;
	}
	if (d[2])
	{
		float rc = 1.0f / d[2];
		dist[2] = (pos1[2] - o[2]) * rc;
		dist[5] = (pos2[2]- o[2]) * rc;
	}

	*distance = std::numeric_limits<float>::max();
	for (int i = 0; i < 6; ++i)
	{
		if (dist[i] > 0)
		{
			ip[i] = o + dist[i] * d;
			if ((ip[i][0] > (pos1[0] - EPSILON)) && (ip[i][0] < (pos2[0] + EPSILON)) &&
				(ip[i][1] > (pos1[1] - EPSILON)) && (ip[i][1] < (pos2[1] + EPSILON)) &&
				(ip[i][2] > (pos1[2] - EPSILON)) && (ip[i][2] < (pos2[2] + EPSILON)))
			{
				if (dist[i] < *distance)
				{
					*distance = dist[i];
					retval = true;
				}
			}
		}
	}
	return retval;
}


float blurt::AxisAlignedBoundingBox::getLongestEdge() const
{
	float dx = pos2[0] - pos1[0];
	float dy = pos2[1] - pos1[1];
	float dz = pos2[2] - pos1[2];

	if (dx > dy)
		return dx > dz ? dx : dz;
	else
		return dy > dz ? dy : dz;
}


