#ifndef HITINFORMATION_H
#define HITINFORMATION_H

/**
 * IntersectionData.h
 *
 * (c) 2007, 2008 Thomas Unterthiner
 *
 */

#include "util/Vector3D.h"

namespace blurt
{

using util::Vector3D;
class TraceableObject;
class Ray;

class IntersectionData
{
	public:
		IntersectionData();
		void testAndSetHit(const TraceableObject* obj, float distance);
		void setHit(const TraceableObject* obj, float distance);
		void reset();

		const TraceableObject* getHitObject() const {return hitObject;}
		bool isIntersection() const {return hitObject != 0;}
		float getDistance() const {return distance;}

	private:
		friend class IntersectionInformation;

		const TraceableObject* hitObject;
		float distance;
};


// this is called in the main-loop of the raytracing-algorithm,
// so it might be worth inlining
inline void IntersectionData::testAndSetHit(const TraceableObject* obj, float distance_)
{
	if (obj == 0 || distance < distance_)
		return;

	setHit(obj, distance_);
}

// gets called a lot, so it's worth inlining :)
inline void IntersectionData::reset()
{
	hitObject = 0;
	distance = std::numeric_limits<float>::max();
}


};

#endif

