#ifndef KDTREEHELPER_H
#define KDTREEHELPER_H

/**
 * KdTreeHelper.h
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include <list>

namespace blurt
{

class AxisAlignedBoundingBox;
class TraceableObject;
class Ray;


namespace kdtree
{
	/// A simple ObjectList (must adhere to the stl-container interface).
	typedef ::std::list<const blurt::TraceableObject*> ObjectList;

	/// A node in the tree.

		/// A node in the tree
		class KdNode
		{
			public:
				KdNode() : objList(0), left(0), right(0) {}
				~KdNode();
				bool isLeaf() const {return !left && !right;}
				float getSplitPosition() const {return splitpos;}
				int getSplitAxis() const {return axis;}
				KdNode* getLeft() const {return left;}
				KdNode* getRight() const {return right;}
				const ObjectList& getObjectList() const {return objList;}

				void addObject(const TraceableObject& o) {objList.push_back(&o);}//objList = new ObjectNode(o, objList);}
				void setSplitPosition(float pos) {splitpos = pos;}
				void setLeft(KdNode* l) {left = l;}
				void setRight(KdNode* r) {right = r;}
				void setSplitAxis(int a) {axis = a;}

				/**
				 * Calculates the entry and exit distance of the ray.
				 * Returns false if the ray doesn't intersect the box.
				 */
				bool intersectRay(const AxisAlignedBoundingBox& bounds,
				                  const Ray& ray, float* near, float* far);

			private:
				ObjectList objList;
				float splitpos;
				KdNode* left;
				KdNode* right;
				int axis;
		};

	typedef unsigned char Axis;


	/// Calculates KdTree-Splits in the spatial median.
	struct SpatialMedianSplitter
	{
		static float getSplitPosition(const AxisAlignedBoundingBox& voxel,
									  const Vector3D& size, Axis a, KdNode* currentNode);
	};


	/// Calculates KdTree-Splits based on the SAH-heuristic
	struct SahSplitter
	{
		static float getSplitPosition(const AxisAlignedBoundingBox& voxel,
									  const Vector3D& size, Axis a, KdNode* currentNode);

		private:
			static float calculateCost(float splitpos, ObjectList l, Axis a, float invSav);
	};
};

};

#endif // KDTREEHELPER_H
