#ifndef MATERIAL_H
#define MATERIAL_H

/**
 * Material.h: stores the material-properties of a primitive
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include <iosfwd>
#include "util/Color.h"

namespace blurt
{

using util::Color;

class Material
{
	public:
		Material(); // NOTE: use operator>> to create a material!
		Material(const Color& c, float dif = 0.2, float spec = 0.8, float phongC = 20,
		         float refl = 0, float refr = 0, float refrIndex = 1.5f,
		         float density = 0.15f);
		~Material();

		float getDiffuse() const {return diffuse;}
		float getSpecular() const {return specular;}
		float getPhongConstant() const {return phongConstant;}
		float getReflective() const {return reflective;}
		float getRefractive() const {return refractive;}
		float getRefractionIndex() const {return refractionIndex;}
		float getDensity() const {return density;}
		const Color& getColor() const {return c;}

		void setDiffuse(float value) {diffuse = value;}
		void setSpecular(float value) {specular = value;}
		void setReflective(float value)  {reflective = value;}
		void setRefractive(float value)  {refractive = value;}
		void setRefractionIndex(float value)  {refractionIndex = value;}
		void setColor(const Color& value) {c = value;}

	private:
		Color c;
		float diffuse;
		float specular;
		float phongConstant;
		float reflective;
		float refractive;
		float refractionIndex;
		float density;

		friend ::std::istream& operator>> (::std::istream& stream, Material& m);
};

::std::ostream& operator<< (::std::ostream& stream, const blurt::Material& m);
::std::istream& operator>> (::std::istream& stream, blurt::Material& m);

};


#endif // MATERIAL_H
