#include "NoSpatialSubdivision.h"
#include "Scene.h"
#include "Ray.h"
#include "IntersectionData.h"
#include "defaultsettings.h"
#include "TraceableObject.h"

#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH

namespace blurt
{

void NoSpatialSubdivision::traceRay(const Ray& ray, IntersectionData* id) const
{
	bool isIntersetion = false;
	float distance;
	foreach (const TraceableObject& o, scene.getObjects())
	{
		isIntersetion = o.intersectsRay(ray, &distance);
		if (isIntersetion)
			id->testAndSetHit(&o, distance);
	}
}


void NoSpatialSubdivision::traceRayPacket(RayPacket* rayPacket) const
{
	foreach (const TraceableObject& o, scene.getObjects())
		o.intersectRayPacket(rayPacket);
}


bool NoSpatialSubdivision::isFirstIntersection(const Ray& ray, float distance) const
{
	bool isIntersetion = false;
	foreach (const TraceableObject& o, scene.getObjects())
	{
		float currentDistance = std::numeric_limits<float>::max();
		isIntersetion = o.intersectsRay(ray, &currentDistance);
		if (isIntersetion &&  currentDistance > 0.f && currentDistance < distance)
			return false;
	}

	return true;
}

}
