#ifndef PARTICLESYSTEM_H
#define PARTICLESYSTEM_H

/**
 * ParticleSystem.h: a simple ParticleSystem.
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "Material.h"
#include "util/Vector3D.h"

#include <list>
#include <iosfwd>

namespace blurt
{

using util::Vector3D;

class Sphere;


class ParticleSystem
{
	public:
		ParticleSystem(const Vector3D& pos, const Vector3D& emitDirection);
		ParticleSystem(std::istream& is);
		~ParticleSystem();

		/// Updates all the particles
		void update();
	private:
		void createParticles();

		struct Particle
		{
			Vector3D pos;
			Vector3D vel;
			Sphere* sphere;
			size_t ttl; // time to live (in frames)
		};

		typedef std::list<Particle> ParticleContainer;
		typedef ParticleContainer::iterator ParticleIter;

		ParticleContainer particles;

		Vector3D position;  // position of the emitter
		Vector3D positionVariance; // variance of the position for new particles
		Vector3D emittingDirection; // direction for new particles
		Vector3D velocityVariance; // variance of the direction for new particles
		Vector3D acceleration;  // accelleration of the particles

		float  ttl; // time to live of new particles (in seconds)
		float spawnrate; // rate of new particles/second
		Material material; // default material for the particles

		friend std::ostream& operator<< (std::ostream& os, const ParticleSystem& ps);
};


std::ostream& operator<< (std::ostream& os, const ParticleSystem& ps);

};

#endif // PARTICLESYSTEM_H
