#include "Plane.h"
#include "Ray.h"
#include "RayPacket.h"
#include "Engine.h"
#include "Scene.h"
#include "util/sse_helper.h"
#include <iostream>

using namespace blurt;
using namespace util;
using namespace util::sse;

blurt::Plane::Plane(const char* name, const Material& material,
             const Vector3D& normal_, float d_)
	: TraceableObject(name, material),
	  normal(normal_), d(d_)
{
	normal.normalize();
}


blurt::Plane::Plane(std::istream& is)
{
	readFromFile(is);
	normal.normalize();
}


blurt::Plane::~Plane()
{
	//dtor
}


void blurt::Plane::writeToFile(std::ostream& os) const
{
	TraceableObject::writeToFile(os);
	os << '\n'
		<< "Normal: " << normal << "\tD: " << d;
}


void blurt::Plane::readFromFile(std::istream& is)
{
	TraceableObject::readFromFile(is);
	is.ignore(9);
	is >> normal;
	is.ignore(3);
	is >> d;
}


bool blurt::Plane::intersectsRayImpl(const Ray& ray, float* distance) const
{
	float dot = dotProduct(normal, ray.getDirection());
	if (dot != 0.0f)
	{
		float dp = dotProduct(normal, ray.getOrigin());
		float dist = -(dp + d) / dot;
		if (dist > 0.0f)
		{
			*distance = dist;
			return true;
		}
	}

	return false;
}


void blurt::Plane::intersectRayPacketImpl(RayPacket* rayPacket) const
{
	const VectorArraySSE& origins = rayPacket->getOrigins();
	const VectorArraySSE& directions = rayPacket->getDirections();
	const VectorArraySSE n(normal[0], normal[1],normal[2]);
	const VectorSSE d(this->d);

	const VectorSSE dot(dotProduct(n, directions));
	const VectorSSE neqMask(compareNeq(dot, ZERO_VEC));

	const VectorSSE dp(dotProduct(n, origins));
	const VectorSSE dist( -(dp + d) / dot);
	const VectorSSE validResults(compareGt(dist, ZERO_VEC) & neqMask);

	rayPacket->updateHitInformation(dist, validResults, this);

}


// taken from the Flipcode-Raytracing tutorial... I actually don't understand it :(
bool blurt::Plane::intersectsBoxImpl(const Vector3D& lower, const Vector3D& higher) const
{
	const Vector3D v[2] = {lower, higher};

	int side1 = 0, side2 = 0;
	for (int i = 0; i < 8; ++i)
	{
		Vector3D p(v[i & 1][0], v[(i >> 1) & 1][1], v[(i >> 2) & 1][2]);
		if ((dotProduct(p, normal ) + d) < 0)
			side1++;
		else
			side2++;
	}

	if ((side1 == 0) || (side2 == 0))
		return false;
	else
		return true;
}


Vector3D blurt::Plane::getNormalImpl(const Vector3D& pos) const
{
	return normal;
}


AxisAlignedBoundingBox blurt::Plane::getAxisAlignedBoundingBoxImpl() const
{
	// it's kind of hard to determine a good AABB for an infinite plane ;)
//	return Engine::get().getScene().getBounds();
	return AxisAlignedBoundingBox(Vector3D(-32, -32, 0), Vector3D(32, 32, 64));
}

