#ifndef RAYPACKET_H
#define RAYPACKET_H
/**
 * RayPacket.h
 *
 * (c) 2007, 2008 Thomas Unterthiner
 *
 * A Packet of Rays. Use this instead of single rays
 * to take advantage of SSE :)
 *
 */

#include "blurt/Ray.h"
#include "blurt/IntersectionData.h"
#include "util/Color.h"
#include "util/sse_helper.h"
#include <algorithm>
#include <boost/array.hpp>

namespace util
{
	class Vector3D;
}

namespace blurt
{

using util::Color;
using util::sse::VectorArraySSE;
using util::sse::VectorSSE;

class TraceableObject;

typedef boost::array<Ray, 4> RayArray;

class RayPacket
{
	public:
		RayPacket() {}
		RayPacket(const RayPacket& other);		RayPacket(const Vector3D* origins, const Vector3D* dirs, size_t numRays);

		Ray getRay(size_t idx);
		RayArray getRays() const;
		const VectorArraySSE& getDirections() const {return directions;}
		const VectorArraySSE& getOrigins() const {return origins;}
		const VectorSSE& getDistances() const {return distances;}
		IntersectionData getIntersectionData(size_t idx) const;

		void updateHitInformation(const VectorSSE& newDists,
		                          const VectorSSE& validDistsMask,
		                          const TraceableObject* obj);

		void setDistances(const VectorSSE& newDist) {distances = newDist;}
		void setHitObject(size_t idx, const TraceableObject* obj) {hitObjects[idx] = obj;}
		void setIntersectionData(size_t idx, float distance, const TraceableObject* hitObject);

		void resetIntersectionData();
		static const size_t PACKET_SIZE = 4;

	private:
		VectorArraySSE directions;
		VectorArraySSE origins;
		VectorSSE distances;
		boost::array<const TraceableObject*, PACKET_SIZE> hitObjects;
};

}
#endif // RAYPACKET_H
