#ifndef SCENE_H
#define SCENE_H

/**
 * Scene.h: a simple scene-manager
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "TraceableObject.h"
#include "AxisAlignedBoundingBox.h"
#include <boost/ptr_container/ptr_vector.hpp>
#include <string>

namespace blurt
{

class Sphere;
class ParticleSystem;
typedef Sphere Light;


class Scene
{
	public:
		typedef boost::ptr_vector<TraceableObject> ObjectContainer;
		typedef boost::ptr_vector<Light> LightContainer;
		typedef boost::ptr_vector<ParticleSystem> ParticleSystemContainer;

		Scene();
		~Scene();

		void update();

		void addObject(TraceableObject* obj);
		void removeObject(TraceableObject* obj);

		void addLight(Light* obj);

		void setCameraPosition(const Vector3D& pos);
		void setViewport(float x, float y, float width, float height);

		const TraceableObject* getObject(const ::std::string& name) const;

		const Vector3D& getCameraPosition() const {return cameraPos;}
		float getViewportX() const {return viewportX;}
		float getViewportY() const {return viewportY;}
		float getViewportWidth() const {return viewportWidth;}
		float getViewportHeight() const {return viewportHeight;}

		const ObjectContainer& getObjects() const {return objects;};
		const LightContainer& getLights() const {return lights;};
		const ParticleSystemContainer& getParticleSystems() const {return particleSystems;};
		ObjectContainer& getObjects() {return objects;};

		void writeToFile(const ::std::string& path) const;
		void readFromFile(const ::std::string& path);

		const AxisAlignedBoundingBox getBounds() const {return bounds;};

	private:
		void recalculateBounds();

		typedef ObjectContainer::iterator ObjectIter;
		ObjectContainer objects;
		LightContainer lights;
		ParticleSystemContainer particleSystems;
		Vector3D cameraPos;

		AxisAlignedBoundingBox bounds;

		// viewport in world coordinates
		float viewportX;
		float viewportY;
		float viewportWidth;
		float viewportHeight;

};

};

#endif // SCENE_H
