#ifndef SPHERE_H
#define SPHERE_H

/**
 * Sphere.h: a sphere-primitive
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "TraceableObject.h"

namespace blurt
{

class Sphere : public TraceableObject
{
	public:
		Sphere(std::istream& is);
		Sphere(const std::string& name, const Material& material,
		       const Vector3D& center, float radius);
		virtual ~Sphere();

		Vector3D& getCenter() {return center;}
		const Vector3D& getCenter() const {return center;}
		void setCenter(const Vector3D& newCenter) {center = newCenter;}

	private:
		virtual bool intersectsRayImpl(const Ray& ray, float* distance) const;
		virtual void intersectRayPacketImpl(RayPacket* rayPacket) const;
		virtual Vector3D getNormalImpl(const Vector3D& pos) const;
		virtual AxisAlignedBoundingBox getAxisAlignedBoundingBoxImpl() const;
		virtual bool intersectsBoxImpl(const Vector3D& lower, const Vector3D& higher) const;
		virtual Type getTypeImpl() const {return SPHERE;}
		virtual void readFromFile(std::istream& is);
		virtual void writeToFile(std::ostream& os) const;

		Vector3D center;
		float radius;
		float radiusSquare; // precalculated square radius
};

};

#endif // SPHERE_H
