#ifndef TRIANGLE_Hconst
#define TRIANGLE_H

/**
 * Triangle.h: a triangle-primitive
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include "TraceableObject.h"

namespace blurt
{

class Triangle : public TraceableObject
{
	public:
		Triangle(const std::string& name, const Material& material,
		         const Vector3D& v1, const Vector3D& v2, const Vector3D& v3);
		Triangle(std::istream& is);
		virtual ~Triangle();

		virtual void readFromFile(std::istream& is);
		virtual void writeToFile(std::ostream& os) const;

		const Vector3D& getVertex(size_t idx) {return v[idx];}

	private:
		virtual bool intersectsRayImpl(const Ray& ray, float* distance) const;
		virtual Vector3D getNormalImpl(const Vector3D& pos) const;
		virtual AxisAlignedBoundingBox getAxisAlignedBoundingBoxImpl() const;
		virtual bool intersectsBoxImpl(const Vector3D& lower, const Vector3D& higher) const;
		virtual Type getTypeImpl() const {return TRIANGLE;}

		bool isPointOnSurface(const Vector3D& point) const;

		void precalculateValues();

		Vector3D v[3];

		// for the plane-equation:
		Vector3D normal;
		float planeD;

		// precalculated values for the intersection tests
		float invDenominator;
		float dot00;
		float dot01;
		float dot11;
};

};



#endif // TRIANGLE_H
