#ifndef FRAMEWORK_H
#define FRAMEWORK_H

/**
 * Framework.h: some utilities to abstract from the underlying
 * GUI-Toolkit :)
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

namespace util
{
	class Vector3D;
	typedef Vector3D Color;
};

class Engine;



namespace platform
{
class Screen;


/**
 * Intializes the framework.
 */
void init();


/**
 * Shuts the framework down.
 */
void shutdown(platform::Screen& s);


/**
 * Returns some kind of time measurement.
 */
unsigned int getTicks();


/**
 * Deals with all kind of input.
 * Returns false if the user requested the program to end.
 */
bool handleInput(const Engine& engine);


/**
 * Pauses the program's execution for a given timeinterval
 */
void sleep(unsigned int milliseconds);


/**
 * Abstracts a screen to draw on.
 */
class Screen
{
	public:
		Screen(int screenWidth, int screenHeight, const char* windowTitle);
		virtual ~Screen();

		virtual void draw(const util::Color* image);
		virtual void save(const char* path);
		virtual void update() const;
		virtual int getWidth() const;
		virtual int getHeight() const;
	private:
		struct ScreenImpl;
		ScreenImpl* d;
};

};



#endif // FRAMEWORK_H
