/**
 * Framework_sdl.cpp:
 *	GLFW-centric implementation of the framework.
 *
 * (c) 2007 Thomas Unterthiner
 *
 */


#include "Framework.h"
#include "blurt/EngineError.h"
#include "util/Color.h"
#include <GL/glfw.h>
#include <GL/gl.h>
#include <iostream>

using blurt::EngineError;

namespace platform
{

/**
 * Intializes the framework.
 */
void init()
{
	if (!glfwInit())
		throw EngineError("unable to initialize GLFW");
	atexit(glfwTerminate);
	glfwEnable( GLFW_STICKY_KEYS );
}


void shutdown(platform::Screen& s)
{
}


/**
 * Returns some kind of time measurement.
 */
unsigned int getTicks()
{
	return (unsigned int) (glfwGetTime() * 1000);
}


/**
 * Deals with all kind of input.
 */
bool handleInput(const Engine& engine)
{
	return !glfwGetKey(GLFW_KEY_ESC) && glfwGetWindowParam(GLFW_OPENED);
}


void sleep(unsigned int milliseconds)
{
	glfwSleep(milliseconds / 1000.0);
}

struct Screen::ScreenImpl
{
};

Screen::Screen(int screenWidth, int screenHeight, const char* windowTitle)
	: d(0)
{
	if (!glfwOpenWindow(screenWidth, screenHeight, 0,0,0,0,0,0, GLFW_WINDOW))
		throw EngineError("unable to set the desired video-mode");
	glfwSetWindowTitle(windowTitle);

	glViewport (0, 0, screenWidth, screenHeight);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, screenWidth, 0, screenHeight, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}


Screen::~Screen()
{
}


void Screen::save(const char* path)
{
	throw EngineError("not implemented in the GLFW-Backend");
}


void Screen::draw(const Color* image)
{
	int w = getWidth();
	int h = getHeight();

	glBegin(GL_POINTS);
	glClear(GL_COLOR_BUFFER_BIT);
	for (int y = 0; y < h; ++y)
	{
		for (int x = 0; x < w; ++x)
		{
			const Color& c = image[y * w + x];
			glColor3f(c[0], c[1], c[2]);
			glVertex2i(x, y);
		}
	}
	glEnd();
}


int Screen::getWidth() const
{
	int w, h;
	glfwGetWindowSize(&w, &h);
	return w;
}


int Screen::getHeight() const
{
	int w, h;
	glfwGetWindowSize(&w, &h);
	return h;
}


void Screen::update() const
{
	glfwSwapBuffers();
};

};


