#ifndef CLOCK_H
#define CLOCK_H

/**
 * Clock.h: a basic clock class
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include <cstddef>

namespace util
{

typedef unsigned int Time;

/**
 * The basic clock of the game.
 * The Clock is responsible for calculating FPS and
 * providing the timing for the game (making sure logic runs at
 * constant logicticks/seconds).
 */
class Clock
{
	public:
		Clock();

		void startFrame();
		void endFrame();

		float getFps() const;
		unsigned getFrameCount() const {return frameCount;}

		/** Returns the time (in ms) the last frame took */
		Time getFrametime() const;


	private:
		Time getTicks() const;
		Time frameStart;
		unsigned int frameCount;

		// for calculating the fps
		// FPS_FRAMES defines how many frames will be considered
		// when calculating the fps
		static const size_t FPS_FRAMES = 4;
		unsigned int frames[FPS_FRAMES];
		size_t frameIdx;
};

};


#endif // CLOCK_H
