#include "CmdLineParser.h"

#include "defaultsettings.h"
#include <iostream>
#include <boost/program_options/options_description.hpp>
#include <boost/program_options/variables_map.hpp>
#include <boost/program_options/parsers.hpp>
#include <boost/program_options/errors.hpp>

namespace po = boost::program_options;
using std::string;
using std::cout;

struct CmdLineParser::CmdLineParserImpl
{
	po::variables_map vm;
	po::options_description visible;

	CmdLineParserImpl(const char* visibleDesc)
		: visible(visibleDesc)
	{}
};

CmdLineParser::CmdLineParser()
	: d(new CmdLineParserImpl("Basic usage: bluRT [inputfile]\n"
		"(if no inputfile is given\"scene.txt\" will be used)\n\nOptions"))
{
}


CmdLineParser::~CmdLineParser()
{
	delete d;
}

bool CmdLineParser::parse(int argc, char** argv)
{
	try
	{
		return parseImpl(argc, argv);
	}
	catch(po::error& e)
	{
		cout << "ERROR: " <<  e.what() << "\n\n";
		cout << d->visible << '\n';
		return false;
	}
}

bool CmdLineParser::parseImpl(int argc, char** argv)
{
	d->visible.add_options()
		("help,h", "produces this help message")
		("anti-aliasing,a", po::value<int>()->default_value(1),
			"anti-aliasing level (1 = no anti-aliasing)")
		("width,w", po::value<int>()->default_value(DEFAULT_SCREENWIDTH),
			"desired imagewidth")
		("height,i", po::value<int>()->default_value(DEFAULT_SCREENHEIGHT),
			"desired imageheight")
		("recursiondepth,r",
			po::value<int>()->default_value(DEFAULT_MAXRECURSIONDEPTH),
			"sets the maximum recursion depth")
		("threads,t", po::value<int>()->default_value(1),
			"number of threads to use")
		("screenshot,s", "makes screenshots of every new generated picture")
		("oneframe,o", "render only one frame")
		("showfps,f", po::value<int>(),
			"display the fps")
		("video,v", po::value<string>(),
			"make a video and store it in arg");


	po::options_description hidden("Hidden options");
	hidden.add_options()
		("data-file", po::value<string>()->default_value(DEFAULT_SCENE_FILE), "");

	po::options_description options;
	options.add(d->visible).add(hidden);

	po::positional_options_description p;
	p.add("data-file", 1);

	po::store(po::command_line_parser(argc, argv).
		  options(options).positional(p).run(), d->vm);
	po::notify(d->vm);

	if (d->vm.count("help"))
	{
		cout << d->visible << "\n";
		return false;
	}
	return true;
}

string CmdLineParser::getArgument(const string& option)
{
	return d->vm[option].as<string>();
}


void CmdLineParser::getArgument(const std::string& option, int* arg)
{
	*arg = d->vm[option].as<int>();
}

bool CmdLineParser::isOptionPresent(const std::string& option)
{
	return d->vm.count(option);
}


