#ifndef VECTOR3D_H
#define VECTOR3D_H

/**
 * Vector3D.h: a basic 3D vector class
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

#include <iosfwd>
#include "mathhelper.h"
#include <xmmintrin.h>


/*
 * NOTE:
 * for some reason, the gcc-vectorisation always segfaults :(
 */
#define USE_GCC_VECTORISATION (defined(__SSE__) && 0)

namespace util
{

class Vector3D
{
	public:
		float& x() {return v[0];}
		float& y() {return v[1];}
		float& z() {return v[2];}
		float& operator[] (size_t idx) 	{return v[idx];}
		float  operator[] (size_t idx) const  	{return v[idx];};

		Vector3D(float x, float y, float y);
		Vector3D(const Vector3D& other);
		Vector3D(__m128 v) : vec(v) {}
		Vector3D();

		float getLength() const;
		void normalize();

		Vector3D& operator-();
		Vector3D& operator*= (float a);
		Vector3D& operator/= (float a);
		Vector3D& operator+= (const Vector3D& other);
		Vector3D& operator-= (const Vector3D& other);
		bool operator==(const Vector3D& other) const;
		bool operator!=(const Vector3D& other) const;

		const float* getAddress(size_t idx) const {return v+idx;}

		friend std::istream& operator>> (std::istream& s, Vector3D& v);

	private:
		union
		{
			__m128 vec;
			float v[4];
		}__attribute__((aligned(16)));


		friend const Vector3D operator*(const Vector3D& lhs, const Vector3D& rhs);
		friend const Vector3D operator+(const Vector3D& lhs, const Vector3D& rhs);
		friend const Vector3D operator-(const Vector3D& lhs, const Vector3D& rhs);
};

Vector3D crossProduct(const Vector3D& lhs, const Vector3D& rhs);
float dotProduct(const Vector3D& lhs, const Vector3D& rhs);
const Vector3D operator*(float lhs, const Vector3D& rhs);
const Vector3D operator*(const Vector3D& lhs, float rhs);
const Vector3D operator/(const Vector3D& lhs, float rhs);
const Vector3D operator*(const Vector3D& lhs, const Vector3D& rhs);
const Vector3D operator+(const Vector3D& lhs, const Vector3D& rhs);
const Vector3D operator-(const Vector3D& lhs, const Vector3D& rhs);


std::ostream& operator<< (std::ostream& stream, const Vector3D& v);
std::istream& operator>> (std::istream& stream, Vector3D& v);


/// Rotates the Vector v arround the axis defined by the origin and the given axis
void rotate(Vector3D* v, const Vector3D& axis, float alpha);


};

#include "Vector3D.cpp"

#endif // VECTOR3D_H
