#ifndef MATHHELPER_H
#define MATHHELPER_H
/**
 * mathhelper.h: some helpful math-functions :)
 *
 * (c) 2007 Thomas Unterthiner
 *
 */

/*
 * Note, to use the fast invsqrt this file MUST
 * be compiled with -fno-strict-aliasing
 * or else GCC will break it when compiling with -O2 or -O3
 */
#define USE_FAST_INVSQRT 0 // tests show a spead DECREASE using it ?-)
#define TWO_ITERATIONS 1
#if !USE_FAST_INVSQRT
	#include <cmath>
#endif


namespace util
{

const float PI = 3.14159265f;


float invsqrt(float x);

/*
 * actually these just wrapps std::powf / std::expf,
 * but i got strange linker-errors when I used them directly
 * ?-)
 */
float mypow(float x, float y);
float myexp(float x);

#if USE_FAST_INVSQRT
/*
 * The famous invsqrt from the Q3A source
 * (with slightly better proposed by Chris Lomont
 */
inline float invsqrt(float x)
{
	float xhalf = 0.5f*x;
	int i = *(int*)&x;
	i = 0x5f375a86 - (i>>1);
	x = *(float*)&i;
	x = x*(1.5f - xhalf*x*x);
	#if TWO_ITERATIONS
	x = x*(1.5f - xhalf*x*x);
	#endif
	return x;
}

#else

inline float invsqrt(float x)
{
	return 1.0f / sqrtf(x);
}

#endif

};

#endif // MATHHELPER_H
