#ifndef VECTORARRAYSSE_H
#define VECTORARRAYSSE_H

/**
 * VectorArraySSE.h: functions to help the use SSE intrinsics
 *
 * (c) 2007, 2008 Thomas Unterthiner
 *
 */

#include "util/Vector3D.h"
#include "util/sse/VectorSSE.h"
#include <boost/array.hpp>


namespace util
{

class Vector3D;

namespace sse
{

/**
 * Contains 3 VectorSSE, usually used to store the x, y and z
 * coordinates of 4 Vector3D.
 * Has overloaded arithmetic operators.
 */
class VectorArraySSE
{
	public:
		VectorArraySSE() {};
		VectorArraySSE(const Vector3D* vec3, size_t numRays);

		VectorArraySSE(const VectorSSE& a, const VectorSSE& b, const VectorSSE& c);
		VectorArraySSE(float x, float y, float z);

		VectorSSE& operator[] (size_t idx) {return v[idx];}
		const VectorSSE&  operator[] (size_t idx) const {return v[idx];}

		void normalize();

	private:
		boost::array<VectorSSE, 3> v;
};

const VectorArraySSE operator*(const VectorArraySSE& lhs, const VectorArraySSE& rhs);
const VectorArraySSE operator+(const VectorArraySSE& lhs, const VectorArraySSE& rhs);
const VectorArraySSE operator-(const VectorArraySSE& lhs, const VectorArraySSE& rhs);

VectorSSE dotProduct(const VectorArraySSE& a, const VectorArraySSE& b);

typedef boost::array<Vector3D, 4> VectorArray3D;

VectorArray3D convertToVector3D(const VectorArraySSE& va);

}}

#include "VectorArraySSE.cpp"


#endif // VECTORARRAYSSE_H
