#ifndef VECTORSSE_H
#define VECTORSSE_H
/**
 * VectorSSE.h: functions to help the use SSE intrinsics
 *
 * (c) 2007, 2008 Thomas Unterthiner
 *
 */


#include <xmmintrin.h>
#include <iosfwd>

namespace util { namespace sse
{


class VectorSSE
{
	public:
		float& x() {return v[0];}
		float& y() {return v[1];}
		float& z() {return v[2];}
		float& operator[] (size_t idx) 	{return v[idx];}
		float  operator[] (size_t idx) const  	{return v[idx];};

		operator __m128() const {return vec;}
		bool isZero(size_t idx) const;
		bool isZero() const;
		void sqrt();

		VectorSSE(float x, float y, float y, float w);
		VectorSSE(float x);
		VectorSSE(const __m128& x) :vec(x) {}
		VectorSSE(const VectorSSE& other);
		VectorSSE();

		VectorSSE& operator+= (const VectorSSE& other);
		VectorSSE& operator-= (const VectorSSE& other);
		VectorSSE& operator*= (const VectorSSE& other);
		VectorSSE& operator&= (const VectorSSE& other);
		VectorSSE& operator|= (const VectorSSE& other);

		union
		{
			__m128 vec;
			float v[4];
			unsigned i[4];
		};
}__attribute__((aligned(16)));


const VectorSSE operator-(const VectorSSE& v);
const VectorSSE operator*(const VectorSSE& lhs, const VectorSSE& rhs);
const VectorSSE operator/(const VectorSSE& lhs, const VectorSSE& rhs);
const VectorSSE operator+(const VectorSSE& lhs, const VectorSSE& rhs);
const VectorSSE operator-(const VectorSSE& lhs, const VectorSSE& rhs);
const VectorSSE operator|(const VectorSSE& lhs, const VectorSSE& rhs);
const VectorSSE operator&(const VectorSSE& lhs, const VectorSSE& rhs);

bool operator==(const VectorSSE& lhs, const VectorSSE& rhs);
bool operator!=(const VectorSSE& lhs, const VectorSSE& rhs);
const VectorSSE compareGt(const VectorSSE& lhs, const VectorSSE& rhs);
const VectorSSE compareLt(const VectorSSE& lhs, const VectorSSE& rhs);
const VectorSSE compareNeq(const VectorSSE& lhs, const VectorSSE& rhs);

std::ostream& operator<< (std::ostream& stream, const VectorSSE& v);

const VectorSSE min(const VectorSSE& a, const VectorSSE& b);

}}

#include "VectorSSE.cpp"

#endif // VECTORSSE_H
